#include <xtl.h>
#include <stdio.h>
#include <string.h>
#include "kernel.h"
#include "types.h"
#include "swap.h"
#include "util.h"
#include "SimpleIni.h"

CSimpleIniA g_ini(1, 0, 0);
BOOL isXex = FALSE;
BOOL iniLoaded = FALSE;

void DeleteLinks(const char* szDrive)
{
	HRESULT res;
	res = deleteLink(szDrive, SYS_STRING);
	res = deleteLink(szDrive, USR_STRING);
}

// find and load multi.ini
BOOL loadIni(void)
{
	int i;
	for(i=0; i < (MOUNT_MAX_ITEMS); i++)
	{
		if(mountPath(MOUNT_NAME, mountPaths[i], USR_STRING) == STATUS_SUCCESS)
		{
			if(fileExists(INI_NAME))
			{
				SI_Error rc = g_ini.LoadFile(INI_NAME);
				if(rc >= 0)
				{
					return TRUE;
				}
			}
		}
	}
	return FALSE;
}

// returns true if info found in ini OK
BOOL getInfoFromIni(DWORD ucNextDisc, DWORD titleId, char* diskpath)
{
	const CHAR* iniPath;
	DWORD devNum = 0xF, testNum = 0, i;
	char dtemp[10];
	char tidtemp[20];
	isXex = FALSE;
	if(!iniLoaded)
	{
		if(loadIni())
			iniLoaded = TRUE;
		else
			return FALSE;
	}
	// try and find the right game in the ini
	sprintf_s(dtemp, 25, "disk%d", ucNextDisc);
	sprintf_s(tidtemp, 25, "%08x", titleId);

	iniPath = g_ini.GetValue(tidtemp, dtemp, NULL);
	if(iniPath == NULL)
		return FALSE;

	// make sure it exists on some device at that path and use that info, if not return FALSE
	// figure out which mount point was used
	for(i = 0; i < DEVS_MAX; i++)
	{
		if(strnicmp(iniPath, numDevs[i].mountPoint, (numDevs[i].stlen+1)) == 0)
		{
			devNum = numDevs[i].startdev;
			testNum = numDevs[i].numdev;
			strcpy_s(diskpath, MAX_PATH, "swp");
			strcat_s(diskpath, MAX_PATH, (iniPath+numDevs[i].stlen));
			if(iniPath[(strlen(iniPath)-1)] == '\\')
			{
				isXex = TRUE;
				strcat_s(diskpath, MAX_PATH, "default.xex");
			}
			i = DEVS_MAX;
		}
	}
	if(devNum != 0xF)
	{
		for(i = 0; i < testNum; i++)
		{
			mount(MOUNT_NAME, mountPaths[(devNum+i)]);
			if(fileExists(diskpath) == TRUE)
			{
				DeleteLinks(MOUNT_NAME);
				// build paths for mounting replacement disk
				if(isXex)
				{
					strcpy_s(diskpath, MAX_PATH, mountPaths[(devNum+i)]);
					strcat_s(diskpath, MAX_PATH, (iniPath+numDevs[i].stlen+2));	
				}
				else
				{
					mountPath(MOUNT_NAME, mountPaths[(devNum+i)], USR_STRING);
					strcpy_s(diskpath, MAX_PATH, (iniPath+numDevs[i].stlen+2));	
				}
				return TRUE;
			}
		}
	}
	return FALSE;
}

DWORD (*XamSwapDiscOrg)(DWORD, PHANDLE, PXSWAPDISC_ERROR_TEXT);
DWORD XamSwapDiscHook(DWORD ucNextDisc, PHANDLE hSwapComplete, PXSWAPDISC_ERROR_TEXT XSwapDiscErrorText)
{
	DWORD ret, titleId;
	CHAR diskpath[MAX_PATH];
	CHAR curpath[MAX_PATH];
	BOOL swapComplete = FALSE;
	titleId = XamGetCurrentTitleId();
	getPath("\\??\\GAME:", curpath);
	DbgPrint("swap disc from tid:%08x called to disk %d\n", titleId, ucNextDisc);
	if(getInfoFromIni(ucNextDisc, titleId, diskpath))
	{
		// cleanup the game link path environment
		if(strnicmp(curpath, "\\Device\\Package_", strlen("\\Device\\Package_")) == 0)
			ret = unmountCon("GAME");
		DeleteLinks("GAME:");
		DeleteLinks("D:");

		if(isXex)
		{
			ret = mountPath("D:", diskpath, USR_STRING);
			ret |= mountPath("GAME:", diskpath, USR_STRING);
			ret |= mountPath("GAME:", diskpath, SYS_STRING);
			if(ret == 0)
				swapComplete = TRUE;
		}
		else
		{
			ret = mountCon("D", MOUNT_NAME, diskpath);
			if(ret == 0)
			{
				// needs to be mounted to "\\??\\D:" , "\\??\\GAME:",  "\\System??\\GAME:"
				getPath("\\??\\D:", diskpath);
				ret = mountPath("GAME:", diskpath, USR_STRING);
				ret |= mountPath("GAME:", diskpath, SYS_STRING);
				if(ret == 0)
					swapComplete = TRUE;
			}
			else
			{
				swapComplete = FALSE;
			}
		}

		if(swapComplete)
		{
			ret = KeSetEvent(hSwapComplete, 1, FALSE);
			ObDereferenceObject(hSwapComplete);
		}
		else
		{
			return XamSwapDiscOrg(ucNextDisc, hSwapComplete, XSwapDiscErrorText);
		}
	}
	else
		return XamSwapDiscOrg(ucNextDisc, hSwapComplete, XSwapDiscErrorText);
	return 0;
}

BOOL APIENTRY DllMain(HANDLE hInstDLL, DWORD reason, LPVOID lpReserved)
{
	switch(reason)
	{
		case DLL_PROCESS_ATTACH:
			XamSwapDiscOrg = (DWORD (*)(DWORD, PHANDLE, PXSWAPDISC_ERROR_TEXT))HookOrd(2600, (DWORD)XamSwapDiscHook, getModuleEat("xam.xex"));
			break;
		case DLL_THREAD_ATTACH:
			break;
		case DLL_THREAD_DETACH:
			break;
		case DLL_PROCESS_DETACH:
			break;
	}
	return TRUE;
}
